import sys
from django.conf import settings
from django.core.management import execute_from_command_line

if not settings.configured:
    settings.configure(
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            },
        },
        INSTALLED_APPS=(
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.messages',
            'django.contrib.admin',
            'rest_framework_fine_permissions',
            'tests'
        ),
        MIDDLEWARE=(
            'django.middleware.common.CommonMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
        ),
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [],
                'APP_DIRS': True,
                'OPTIONS': {
                    'context_processors': [
                        'django.contrib.auth.context_processors.auth',
                        'django.template.context_processors.debug',
                        'django.template.context_processors.i18n',
                        'django.template.context_processors.media',
                        'django.template.context_processors.static',
                        'django.template.context_processors.tz',
                        'django.contrib.messages.context_processors.messages',
                        'django.template.context_processors.request',
                    ],
                },
            },
        ],
        ROOT_URLCONF='tests.urls',
        USE_TZ=False,
        SECRET_KEY='foobar',
        DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
        # SILENCED_SYSTEM_CHECKS=['1_7.W001'],
    )


def runtests():
    argv = sys.argv[:1] + ['test'] + sys.argv[1:]
    execute_from_command_line(argv)


if __name__ == '__main__':
    runtests()



import json

from django.contrib.messages.storage.cookie import (
    CookieStorage, MessageDecoder, MessageEncoder,
)

from django.contrib.messages import get_messages
from django.contrib import messages

from django.core.files.uploadedfile import SimpleUploadedFile
from django.http import HttpRequest
from django.test import TestCase

from rest_framework_fine_permissions.models import UserFieldPermissions
from . import utils


class TestViews(TestCase):

    """ Test views. """

    def setUp(self):
        self.request = HttpRequest()
        self.superuser = utils.create_superuser()
        self.user = utils.create_user()
        self.storage_class = CookieStorage

    def _add_field_perms(self, app, model, *field_names):
        for field_name in field_names:
            utils.add_field_permission(self.user, app,
                                       model, field_name)

    def _get_messages(self, response):
        storage = self.storage_class(response)
        cookie = response.cookies.get(storage.cookie_name)
        if not cookie or cookie['max-age'] == 0:
            return []

        return storage._decode(cookie.value)


    def test_permissions_export_json_unauthorized(self):
        self._add_field_perms('tests', 'account', 'id', 'user')
        self.client.login(username='test', password='pass')
        ufp = UserFieldPermissions.objects.get(user=self.user)

        response = self.client.get('/drffp/export/%s/' % ufp.pk)
        self.assertEqual(response.status_code, 302)

    def test_permissions_export_json_without_permissions(self):
        self.client.login(username='supertest', password='pass')

        response = self.client.get('/drffp/export/%s/' % 1000)
        self.assertEqual(response.status_code, 404)

    def test_permissions_export_json(self):
        self._add_field_perms('tests', 'account', 'id', 'user')
        self.client.login(username='supertest', password='pass')
        ufp = UserFieldPermissions.objects.get(user=self.user)

        response = self.client.get('/drffp/export/%s/' % ufp.pk)
        json_response = json.loads(response.content.decode('utf-8'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response['username'], 'test')
        self.assertListEqual(
            json_response['fields_permissions'],
            [
                {'app_label': 'tests', 'model': 'account', 'name': 'id'},
                {'app_label': 'tests', 'model': 'account', 'name': 'user'},
            ]
        )

    def test_permissions_import_json_missing_file(self):
        self.client.login(username='supertest', password='pass')
        self._add_field_perms('tests', 'account', 'expired_date')
        ufp = UserFieldPermissions.objects.get(user=self.user)

        response = self.client.post('/drffp/import/%s/' % ufp.pk)

        messages = self._get_messages(response)
        self.assertEqual(len(messages), 1)
        self.assertTrue('Missing imported file' in format(messages[0].message))

        self.assertEqual(response.status_code, 302)

    def test_permissions_import_json_wrong_user(self):
        self.client.login(username='supertest', password='pass')
        self._add_field_perms('tests', 'account', 'expired_date')
        ufp = UserFieldPermissions.objects.get(user=self.user)
        permissions_str = """
        {
            "username": "supertest",
            "fields_permissions": [
                {"app_label": "tests", "model": "account", "name": "id"},
                {"app_label": "tests", "model": "account", "name": "user"}
            ]
        }"""

        f = SimpleUploadedFile('file.txt', permissions_str.encode('utf-8'),
                               content_type='application/json')
        response = self.client.post('/drffp/import/%s/' % ufp.pk,
                                    {'perms_upload': f})

        self.assertEqual(response.status_code, 302)

        messages = self._get_messages(response)
        self.assertEqual(len(messages), 2)

        self.assertTrue('The wrong user is defined in the imported file' in format(messages[0].message))
        self.assertTrue('Permissions imported' in format(messages[1].message))


    def test_permissions_import_json_exception(self):
        self.client.login(username='supertest', password='pass')
        self._add_field_perms('tests', 'account', 'expired_date')
        ufp = UserFieldPermissions.objects.get(user=self.user)
        permissions_str = 'empty'

        f = SimpleUploadedFile('file.txt', permissions_str.encode('utf-8'),
                               content_type='application/json')
        response = self.client.post('/drffp/import/%s/' % ufp.pk,
                                    {'perms_upload': f})

        self.assertEqual(response.status_code, 302)

        messages = self._get_messages(response)
        self.assertEqual(len(messages), 1)
        self.assertTrue('Error in the import' in format(messages[0].message))

    def test_permissions_import_json_with_user_field_permissions(self):
        self.client.login(username='supertest', password='pass')
        self._add_field_perms('tests', 'account', 'expired_date')
        ufp = UserFieldPermissions.objects.get(user=self.user)
        permissions_str = """
        {
            "username": "test",
            "fields_permissions": [
                {"app_label": "tests", "model": "account", "name": "id"},
                {"app_label": "tests", "model": "account", "name": "user"}
            ]
        }"""

        f = SimpleUploadedFile('file.txt', permissions_str.encode('utf-8'),
                               content_type='application/json')
        response = self.client.post('/drffp/import/%s/' % ufp.pk,
                                    {'perms_upload': f})
        ufp = UserFieldPermissions.objects.get(user=self.user)
        permissions = ufp.permissions.all()

        self.assertEqual(response.status_code, 302)

        messages = self._get_messages(response)
        self.assertEqual(len(messages), 1)
        self.assertTrue('Permissions imported' in format(messages[0].message))

        self.assertEqual(len(permissions), 2)
        self.assertListEqual(
            [str(p) for p in permissions],
            ['tests | account | id', 'tests | account | user']
        )

    def test_permissions_import_json_without_user_field_permissions(self):
        self.client.login(username='supertest', password='pass')
        permissions_str = """
        {
            "username": "test",
            "fields_permissions": [
                {"app_label": "tests", "model": "account", "name": "id"},
                {"app_label": "tests", "model": "account", "name": "user"}
            ]
        }"""
        f = SimpleUploadedFile('file.txt', permissions_str.encode('utf-8'),
                               content_type='application/json')
        response = self.client.post('/drffp/import/', {'perms_upload': f})
        ufp = UserFieldPermissions.objects.get(user=self.user)
        permissions = ufp.permissions.all()

        self.assertEqual(response.status_code, 302)

        messages = self._get_messages(response)
        self.assertEqual(len(messages), 1)
        self.assertTrue('Permissions imported' in format(messages[0].message))

        self.assertEqual(len(permissions), 2)
        self.assertListEqual(
            [str(p) for p in permissions],
            ['tests | account | id', 'tests | account | user']
        )




from rest_framework_fine_permissions import serializers
from rest_framework_fine_permissions import fields
from rest_framework import serializers as drf_serializers

from .models import Account, Card, Service


class ServiceSerializer(serializers.ModelPermissionsSerializer):

    class Meta:
        model = Service
        fields = '__all__'


class AccountSerializer(serializers.ModelPermissionsSerializer):

    is_expired = drf_serializers.BooleanField(source='is_expired',
                                              read_only=True)
    full_name = drf_serializers.CharField(source='full_name', read_only=True)
    cards = fields.ModelPermissionsField('tests.CardSerializer')
    service_names = fields.ModelPermissionsField(ServiceSerializer, source='services')

    class Meta:
        depth = 1
        model = Account
        exclude = ('id',)


class CardSerializer(serializers.ModelPermissionsSerializer):

    account = fields.ModelPermissionsField(AccountSerializer)
    service_names = fields.ModelPermissionsField(ServiceSerializer, source='services')

    class Meta:
        model = Card
        fields = '__all__'


class AnotherCardSerializer(serializers.ModelPermissionsSerializer):

    account = fields.ModelPermissionsField(AccountSerializer)
    service_names = fields.ModelPermissionsField('tests.ServiceSerializer', source='services')

    class Meta:
        model = Card
        fields = '__all__'


class BadCardSerializer(serializers.ModelPermissionsSerializer):

    account_name = fields.ModelPermissionsField(AccountSerializer)

    class Meta:
        model = Card
        fields = '__all__'


class TestNormalSerializer(drf_serializers.ModelSerializer):
    pass



from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from rest_framework_fine_permissions import models

from .models import Card, Account, Service, TestUser


def create_user(username='test', password='pass', **kwargs):
    """ Create a user for tests. """
    return User.objects.create_user(username, password=password, **kwargs)


def create_superuser(username='supertest', email='supertest@test.com',
                     password='pass', **kwargs):
    """ Create a user for tests. """
    return User.objects.create_superuser(username, email, password, **kwargs)


def create_account(user):
    """ Create an account for tests. """
    from datetime import datetime
    test_user = TestUser.objects.create(username=user.username)
    return Account.objects.create(user=test_user, expired_date=datetime.now())


def create_card(account):
    """ Create a card for tests. """
    return Card.objects.create(account=account)


def create_service(name):
    """ Create a service for tests. """
    return Service.objects.create(name=name)


def add_field_permission(user, app_label, model_name, field_name):
    """ Add permissions for field on an app model. """
    ct = ContentType.objects.get_by_natural_key(app_label, model_name)
    fp = models.FieldPermission.objects.create(content_type=ct,
                                               name=field_name)
    ufp = models.UserFieldPermissions.objects.get_or_create(user=user)[0]
    ufp.permissions.add(fp)


def remove_all_field_permissions(user):
    """ Remove all field permissions for user. """
    ufp = models.UserFieldPermissions.objects.filter(user=user)
    ufp.delete()



from django.test import TestCase
from django.http import HttpRequest
from rest_framework.exceptions import ValidationError
from rest_framework_fine_permissions.fields import (
    get_serializer, ModelPermissionsField)

from . import models
from . import serializers
from . import utils


class TestModelFieldPermissions(TestCase):

    """ Test a declared field that have permissions on subfields. """

    def setUp(self):
        self.Serializer = serializers.CardSerializer
        self.request = HttpRequest()
        self.user = utils.create_user()

        self.test_user = utils.create_user('test_data')
        self.account = utils.create_account(self.test_user)
        self.card = utils.create_card(self.account)

    def tearDown(self):
        utils.remove_all_field_permissions(self.user)
        self.user.delete()

        self.test_user.delete()
        self.account.delete()
        self.card.delete()

        del self.request

    def _add_field_perms(self, app, model, *field_names):
        for field_name in field_names:
            utils.add_field_permission(self.user, app,
                                       model, field_name)

    def _auth_user(self):
        self.request.user = self.user

    def _get_serializer_instance(self, instance=None, anonymous=False):
        if anonymous:
            self.Serializer(instance=instance, context={})
        self._auth_user()
        return self.Serializer(instance=instance,
                               context={'request': self.request})

    def test_simple_data_in_permissions_field(self):
        """ Test with a many to one field. """
        self._add_field_perms('tests', 'account', 'user')
        self._add_field_perms('tests', 'card', 'account')

        ser = self._get_serializer_instance(instance=self.card)

        data = ser.data

        self.assertEqual(sorted(data['account']), ['user'])
        self.assertEqual(data['account']['user']['id'], self.account.id)

    def test_list_data_in_permissions_field(self):
        """ Test with a many to one field. """
        self._add_field_perms('tests', 'account', 'id', 'user')
        self._add_field_perms('tests', 'card', 'id', 'service_names')
        self._add_field_perms('tests', 'service', 'name', 'id')

        service = utils.create_service('chronos')
        service2 = utils.create_service('moodle')
        self.card.services.add(service)
        self.card.services.add(service2)

        ser = self._get_serializer_instance(instance=self.card)
        data = ser.data

        self.assertEqual(sorted(data['service_names'][0]), ['id', 'name'])
        self.assertEqual(len(data['service_names']), 2)
        self.assertEqual(data['service_names'][0]['name'], 'chronos')

    def test_defined_with_no_dependency(self):
        """ Test with serializer passed as a string. """
        self._add_field_perms('tests', 'card', 'account', 'service_names')
        self.Serializer = serializers.AnotherCardSerializer

        ser = self._get_serializer_instance()
        service_names = ser.get_fields()['service_names']
        self.assertEqual(service_names.serializer,
                         serializers.ServiceSerializer)

    def test_to_internal_value(self):
        field = ModelPermissionsField(
            serializers.AccountSerializer,
            queryset=models.Account.objects.all()
        )
        instance = field.to_internal_value(self.account.pk)
        self.assertEqual(instance.pk, self.account.pk)

    def test_to_internal_value_does_not_exist(self):
        field = ModelPermissionsField(
            serializers.AccountSerializer,
            queryset=models.Account.objects.all()
        )
        with self.assertRaises(ValidationError) as ve:
            field.to_internal_value(0)
        msg = ve.exception.detail[0]

        self.assertEqual('Object with pk=0 does not exist.', msg)


class TestLoadSerializer(TestCase):

    """ Test loading serializer from string. """

    def test_direct_to_serializer(self):
        """ Test with no operation to do. """
        ser = get_serializer(serializers.AccountSerializer)
        self.assertEqual(ser, serializers.AccountSerializer)

    def test_serializer_found(self):
        """ Test founding a serializer from an app label and his name. """
        ser = get_serializer('tests.AccountSerializer')
        self.assertEqual(ser, serializers.AccountSerializer)

    def test_serializer_not_found(self):
        """ Test not founding a serializer from an app label and his name. """
        ser = get_serializer('notests.AccountSerializer')
        self.assertIsNone(ser)



#!/usr/bin/env python

from django.core.management.base import BaseCommand, CommandError
from rest_framework_fine_permissions.models import UserFieldPermissions
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
import json


class Command(BaseCommand):

    help = ("Output the user's fieds permissions of the database as a fixture of the given user ")
    args = 'user'

    def handle(self, *args, **options):
        """
        dump fields permissions for a user
        """

        def get_user(username):
            try:
                return User.objects.get(username=username)
            except ObjectDoesNotExist as e:
                raise CommandError("This user doesn't exist in the database")

        def get_fields_permissions(user):
            try:
                return UserFieldPermissions.objects.get(user=user)
            except ObjectDoesNotExist as e:
                raise CommandError("This user doesn't have fields permissions")



        if len(args) !=1:
            raise CommandError("Specifies a user")
        else:
            try:
                self.stdout.ending = None
                user = get_user(args[0])
                fields_permissions  = get_fields_permissions(user)
                permissions = fields_permissions.permissions.all()

                jsoneatthat = {'username': user.username, 
                    'fields_permissions' : [ {'app_label': permission.content_type.app_label, 'model' : permission.content_type.model, 'name': permission.name } for permission in permissions]}

                self.stdout.write(json.dumps(jsoneatthat))

            except:
                raise




# -*- coding: utf-8 -*-

"""
"""

from django import forms
from django.contrib import admin
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.utils.html import format_html

from .fields import ModelPermissionsField
from .models import (
    FieldPermission, UserFieldPermissions, FilterPermissionModel)
from .utils import get_field_permissions
from .serializers import QSerializer


class UserFieldPermissionsForm(forms.ModelForm):

    """
    """

    permissions = forms.MultipleChoiceField(
        widget=FilteredSelectMultiple(
            verbose_name='User field permissions',
            is_stacked=False,
        )
    )

    class Meta:
        model = UserFieldPermissions
        exclude = ('content_type',)

    def __init__(self, *args, **kwargs):
        super(UserFieldPermissionsForm, self).__init__(*args, **kwargs)

        # Choices
        def choice_str(model, field, sep):
            return '{model[0]}{sep}{model[1]}{sep}{field}'\
                .format(model=model.split('.'), field=field, sep=sep)

        choices = []
        self.field_permissions = get_field_permissions()
        self.field_serializers = {}

        for model, values in self.field_permissions.items():
            fields, ser = values
            choices += [(
                choice_str(model, field, '.'),
                choice_str(model, field, ' | '))
                for field in fields]
            self.field_serializers.update(
                {choice_str(model, field, '.'): ser for field in fields})
        self.fields['permissions'].choices = sorted(choices)

        # Initial datas
        instance = kwargs.get('instance')
        if instance:
            fps = ['.'.join(fp) for fp in instance.permissions.all()
                   .values_list(
                       'content_type__app_label',
                       'content_type__model',
                       'name')
                   ]
            self.initial['permissions'] = fps

    def clean(self):
        cleaned = self.cleaned_data
        field_permissions = []
        model_perms = {}
        conflicts = []

        for permission in cleaned.get('permissions', []):
            app_label, model_name, field_name = permission.split('.')
            ct = ContentType.objects.get_by_natural_key(app_label, model_name)
            fp = FieldPermission.objects.get_or_create(
                content_type=ct,
                name=field_name
            )[0]
            field_permissions.append(fp.pk)

            # Check for each added permission if there is a recursive call
            # between two ModelPermissionsField fields
            app = '%s.%s' % (app_label, model_name)
            values = self.field_permissions.get(app)
            fields = values[0] if values else []

            if field_name in fields:
                field = self.field_permissions.get(app)
                field = field[0][field_name] if field else None

                if isinstance(field, ModelPermissionsField):
                    name = '{0.__module__}.{0.__name__}'.format(
                        field.serializer)
                    model_perms[self.field_serializers[permission]] = permission

                    if name in model_perms:
                        conflicts.append(
                            '<li><b>%s</b> and <b>%s</b></li>'
                            % (permission, model_perms[name]))

        if conflicts:
            msg = 'Recursive ModelPermissionsField call between<ul>{}</ul>'\
                .format(''.join(conflicts))
            raise forms.ValidationError(format_html(msg))

        self.cleaned_data['permissions'] = field_permissions


class UserFieldPermissionsAdmin(admin.ModelAdmin):

    """
    """
    list_display = ('user', )
    form = UserFieldPermissionsForm
    ordering = ('user__username',)


class ContentTypeChoiceField(forms.ModelChoiceField):
    def label_from_instance(self, obj):
        return "%s | %s" % (obj.app_label, obj.model)


class UserFilterPermissionsForm(forms.ModelForm):
    """
    filter permissions form
    """

    current_filter = forms.CharField(required=False)
    content_type = ContentTypeChoiceField(
        queryset=ContentType.objects.all().order_by('app_label', 'model'))

    class Meta:
        model = FilterPermissionModel
        exclude = []

    def __init__(self, *args, **kwargs):
        super(UserFilterPermissionsForm, self).__init__(*args, **kwargs)

        # Initial datas
        instance = kwargs.get('instance')
        self.fields['current_filter'].widget.attrs['readonly'] = True
        self.fields['current_filter'].widget.attrs['size'] = 130

        if instance:
            myq = QSerializer(base64=True).loads(instance.filter)
            current_filter = myq.__str__()
        else:
            myq = Q(myfield="myvalue")
            current_filter = ""

        self.initial['current_filter'] = current_filter
        self.initial['filter'] = QSerializer().dumps(myq)

        self.fields['filter'].widget.attrs['rows'] = len(
            self.initial['filter'].splitlines()) + 4

    def clean_filter(self):
        data = self.cleaned_data['filter']
        if data:
            try:
                myq = QSerializer().loads(data)
            except Exception:
                raise forms.ValidationError(
                    "filter is not a valid entry for a q object !")
            else:
                if isinstance(myq, Q):
                    data = QSerializer(base64=True).dumps(myq)
                else:
                    raise forms.ValidationError("filter is not a q object !")

        return data

    def save(self, commit=True):
        form = super(UserFilterPermissionsForm, self).save(commit=False)

        if commit:
            form.save()
            form.save_m2m()

        return form


class UserFilterPermissionsAdmin(admin.ModelAdmin):
    """
    filter permissions admin
    """
    list_display = ('user', 'content_type')
    form = UserFilterPermissionsForm


admin.site.register(UserFieldPermissions, UserFieldPermissionsAdmin)
admin.site.register(FilterPermissionModel, UserFilterPermissionsAdmin)



# -*- coding: utf-8 -*-

"""
"""

from importlib import import_module
import inspect
from collections import OrderedDict
import logging

from django.apps import apps
from django.conf import settings
from rest_framework.utils.model_meta import get_field_info
from six import iterkeys, string_types
from itertools import chain


logger = logging.getLogger(__name__)
APP_NAMES = None


def get_application(app_label):
    """ Get an application. """
    app_config = apps.get_app_config(app_label)
    return app_config.module.__name__


def get_serializer(serializer):
    """ Load a serializer. """
    if isinstance(serializer, string_types):
        try:
            app_label, serializer_name = serializer.split('.')
            app_package = get_application(app_label)
            serializer_module = import_module('%s.serializers' % app_package)
            serializer = getattr(serializer_module, serializer_name)
        except Exception as e:
            logger.error('Serializer %s not found: %s' % (serializer, e))
            return None
    return serializer


def inherits_modelpermissions_serializer(cls):
    """ Verify that serializer is a :py:class:`~rest_framework_fine_permissions.serializers.ModelPermissionsSerializer`. """
    def is_modelperms():
        return 'ModelPermissionsSerializer' in [
            ser.__name__ for ser in cls.__bases__
        ]
    is_serializer = type(cls).__name__ == 'SerializerMetaclass'

    return inspect.isclass(cls) and is_serializer and is_modelperms()


def merge_fields_and_pk(pk, fields):
    fields_and_pk = OrderedDict()
    fields_and_pk[pk.name] = pk
    fields_and_pk.update(fields)
    return fields_and_pk


def get_model_fields(model):
    fields_info = get_field_info(model)
    return chain(iterkeys(merge_fields_and_pk(fields_info.pk,
                                              fields_info.fields)),
                 iterkeys(fields_info.relations))


def get_permitted_fields(model, serializer):
    fields = get_model_fields(model)
    result = dict.fromkeys(fields, None)
    result.update(serializer._declared_fields)
    exclude = getattr(serializer.Meta, 'exclude', ())

    for field in exclude:
        result.pop(field, None)

    return result


def get_field_permissions():
    serializers = {}

    for app in settings.INSTALLED_APPS:
        try:
            mod = import_module('%s.serializers' % app)
            for obj_name in dir(mod):
                obj = getattr(mod, obj_name)

                # Check if it is a serializer subclass
                if inherits_modelpermissions_serializer(obj):
                    try:
                        model = obj.Meta.model
                        fields = get_permitted_fields(model, obj)
                        perm_key = '{0.app_label}.{0.model_name}'\
                            .format(model._meta)
                        serializers[perm_key] = (
                            fields, '{0.__module__}.{0.__name__}'.format(obj))
                    except AttributeError:
                        pass

        except ImportError:
            continue

    return serializers



from django.test import TestCase
from rest_framework_fine_permissions import utils
from . import serializers as test_serializers
from .models import Account


class TestUtilities(TestCase):

    """ Test utility functions. """

    def setUp(self):
        self.account_ser = test_serializers.AccountSerializer
        self.normal_ser = test_serializers.TestNormalSerializer

    def test_inherits(self):
        """ Test inherits from ModelPermissionsSerializer. """
        self.assertTrue(
            utils.inherits_modelpermissions_serializer(self.account_ser)
        )

    def test_not_inherits(self):
        """ Test don't inherit from ModelPermissionsSerializer. """
        self.assertFalse(
            utils.inherits_modelpermissions_serializer(self.normal_ser)
        )

    def test_model_retrieving_model_fields(self):
        """ Test field names extracted from the model. """
        model_fields = ('id', 'user', 'expired_date', 'cards')
        self.assertEqual(set(utils.get_model_fields(Account)),
                         set(model_fields))

    def test_permitted_fields(self):
        """ Test list of fields displayed in admin interface. """
        permit_fields = utils.get_permitted_fields(Account, self.account_ser)
        model_fields = ('user', 'expired_date', 'cards', 'service_names')
        serializer_fields = ('is_expired', 'full_name')
        self.assertEqual(
            set(permit_fields.keys()), set(model_fields + serializer_fields))

    def test_field_permissions(self):
        """ Test retrieving permissions by application. """
        permissions = utils.get_field_permissions()
        self.assertTrue('tests.account' in permissions)
        self.assertEqual(
            set(permissions['tests.account'][0].keys()),
            {
                'user', 'expired_date', 'cards', 'is_expired', 'expired_date',
                'full_name', 'service_names'
            })



import collections
from django.test import TestCase
from django.http import HttpRequest
from rest_framework.utils.model_meta import get_field_info

from rest_framework_fine_permissions import fields

from . import utils
from . import serializers


class ModelSerializerTestMixin(object):

    """ Mixin for testing Serializer. """

    def _auth_user(self):
        self.request.user = self.user

    def _add_field_perms(self, app, model, *field_names):
        for field_name in field_names:
            utils.add_field_permission(self.user, app,
                                       model, field_name)

    def _get_model_info(self):
        return get_field_info(self.Serializer.Meta.model)

    def _get_relation_info(self, relation_field_name):
        return self._get_model_info().relations[relation_field_name]

    def _get_serializer_instance(self, anonymous=False):
        if anonymous:
            self.Serializer(context={})
        self._auth_user()
        return self.Serializer(context={'request': self.request})


class TestModelSerializer(TestCase, ModelSerializerTestMixin):

    """ Test ModelPermissionSerializer inheriting. """

    def setUp(self):
        self.Serializer = serializers.AccountSerializer
        self.user = utils.create_user('test')
        self.request = HttpRequest()

    def tearDown(self):
        utils.remove_all_field_permissions(self.user)
        self.user.delete()
        del self.request

    def test_no_request_to_instantiate(self):
        """ Test instantiation without request in context. """
        ser = self.Serializer()
        self.assertIsNone(ser.user)

    def test_default_field_names(self):
        """ Test default field names that can be accessed. """
        ser = self._get_serializer_instance()
        self.assertEqual(set(ser._get_default_field_names(ser._declared_fields,
                             self._get_model_info())),
                         {'id', 'expired_date', 'is_expired', 'cards', 'user',
                          'full_name', 'service_names'})

    def test_nested_class(self):
        """ Test retrieving nested relation. """
        ser = self._get_serializer_instance()
        user_relation = self._get_relation_info('user')
        user_nested_class = ser._get_nested_class(1, user_relation)
        self.assertEqual(user_nested_class.__name__,
                         'NestedModelPermissionSerializer')

    def test_depth_in_nested_class(self):
        """ Test value of depth in nested class' Meta options . """
        ser = self._get_serializer_instance()
        depth = 2
        user_relation = self._get_relation_info('user')
        user_nested_class = ser._get_nested_class(depth, user_relation)
        self.assertEqual(user_nested_class.Meta.depth, 1)

    def test_model_in_nested_class(self):
        """ Test value of model in nested class' Meta options . """
        ser = self._get_serializer_instance()
        user_relation = self._get_relation_info('user')
        user_nested_class = ser._get_nested_class(1, user_relation)

        from .models import TestUser
        self.assertEqual(user_nested_class.Meta.model, TestUser)

    def test_rights_on_nested_field_but_no_subfields(self):
        """ Test default building of relations with no rights on subfields. """
        self._add_field_perms('tests', 'account', 'user')
        ser = self._get_serializer_instance()
        fields = ser.get_fields()
        self.assertEqual(fields['user'].__class__.__name__, 'NestedSerializer')

    def test_allowed_fields_with_no_rights(self):
        """ Test allowed fields with no rights defined for user. """
        ser = self._get_serializer_instance()
        fields = ser._get_user_allowed_fields()
        self.assertEqual(set(fields), set())

    def test_allowed_fields_with_some_field_rights(self):
        """ Test allowed fields with rights defined for user. """
        self._add_field_perms('tests', 'account', 'id', 'user', 'is_expired',
                              'cards')
        ser = self._get_serializer_instance()
        fields = [field.name for field in ser._get_user_allowed_fields()]
        self.assertEqual(set(fields), {'user', 'is_expired', 'id', 'cards'})

    def test_get_field_for_superuser(self):
        """ Test getting all fields for a superuser. """
        self.user.is_superuser = True
        self.user.save()
        ser = self._get_serializer_instance()
        fields = ser.get_fields()
        self.assertIsInstance(fields, collections.OrderedDict)
        self.assertEqual(
            set(fields), {'user', 'is_expired', 'cards', 'full_name',
                          'expired_date', 'service_names'})

    def test_get_field_for_anonymous_user(self):
        """ Test getting all fields for an anonymous user. """
        ser = self._get_serializer_instance(anonymous=True)
        fields = ser.get_fields()
        self.assertIsInstance(fields, collections.OrderedDict)
        self.assertEqual(set(fields), set())

    def test_get_field_for_authenticated_user(self):
        """ Test getting fields for an authenticated user. """
        self._add_field_perms('tests', 'account', 'user')
        ser = self._get_serializer_instance()
        fields = ser.get_fields()
        self.assertIsInstance(fields, collections.OrderedDict)
        self.assertEqual(set(fields), {'user'})


class TestNestedRelations(TestCase, ModelSerializerTestMixin):

    """ Test inner relations representation. """

    def setUp(self):
        self.Serializer = serializers.AccountSerializer
        self.user = utils.create_user('test')
        self.request = HttpRequest()

    def tearDown(self):
        utils.remove_all_field_permissions(self.user)
        self.user.delete()
        del self.request

    def _get_nested_field(self, serializer, field_name):
        relation = self._get_relation_info(field_name)
        nested_class = serializer._get_nested_class(1, relation)
        return nested_class()

    def test_context_initialisation(self):
        """ Test that context is set in a good way. """
        ser = self._get_serializer_instance()
        user_nested_field = self._get_nested_field(ser, 'user')
        self.assertEqual(user_nested_field.context, ser.context)

    def test_no_rights_on_nested_subfields(self):
        """ Test with no rights on subfields. """
        ser = self._get_serializer_instance()
        user_nested_field = self._get_nested_field(ser, 'user')
        fields = user_nested_field.get_fields()
        self.assertEqual(set(fields), set())

    def test_with_rights_on_nested_subfields(self):
        """ Test with rights on subfields. """
        self._add_field_perms('tests', 'account', 'id', 'user')
        self._add_field_perms('tests', 'testuser', 'username')
        ser = self._get_serializer_instance()
        user_nested_field = self._get_nested_field(ser, 'user')
        fields = user_nested_field.get_fields()
        self.assertEqual(set(fields), {'username'})


class TestModelPermissionsField(TestCase, ModelSerializerTestMixin):

    """ Test with a declared field that have permissions. """

    def setUp(self):
        self.Serializer = serializers.CardSerializer
        self.user = utils.create_user('test')
        self.request = HttpRequest()

    def tearDown(self):
        utils.remove_all_field_permissions(self.user)
        self.user.delete()
        del self.request

    def test_permissions_field_instance(self):
        """ Test permission field has right instance. """
        self._add_field_perms('tests', 'card', 'id', 'account')
        ser = self._get_serializer_instance()
        account = ser.get_fields()['account']
        self.assertIsInstance(account, fields.ModelPermissionsField)

    def test_permissions_field_names(self):
        """ Test field that can be accessed. """
        self._add_field_perms('tests', 'card', 'id', 'account')
        self._add_field_perms('tests', 'account', 'full_name')
        ser = self._get_serializer_instance()
        account_ser = ser.get_fields()['account'].serializer(
            context={'request': self.request}
        )
        self.assertEqual(set(account_ser.get_fields()), {'full_name'})



# -*- coding: utf-8 -*-

"""
"""
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _


class FieldPermission(models.Model):
    name = models.CharField(_('name'), max_length=255)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)

    class Meta:
        verbose_name = _('field permission')
        verbose_name_plural = _('fields permissions')
        db_table = 'drf_field_permission'

    def __str__(self):
        return '{0.content_type.app_label} | {0.content_type.model} | {0.name}'\
            .format(self)


class UserFieldPermissions(models.Model):
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    permissions = models.ManyToManyField(
        FieldPermission,
        related_name='user_field_permissions',
        db_table='drf_user_field_permissions_permission')

    class Meta:
        verbose_name = _('user field permission')
        verbose_name_plural = _('user fields permissions')
        db_table = 'drf_user_field_permissions'

    def __str__(self):
        return self.user.username


class FilterPermissionModel(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    filter = models.TextField()

    class Meta:
        verbose_name = _('user filter permission')
        verbose_name_plural = _('user filters permissions')
        db_table = 'drf_user_filter_permissions'
        unique_together = ('user', 'content_type',)

    def __str__(self):
        return '{0.content_type.app_label} | {0.content_type.model} | {0.user.username} | {0.filter}'\
            .format(self)



from django.conf.urls import url

from . import views

urlpatterns = [
    # Export permissions
    url(r'^drffp/export/(?P<ufp_id>\d+)/$', views.permissions_export_json, name='drffp-export'),

    # Import permissions
    url(r'^drffp/import/$', views.permissions_import_json, name='drffp-import'),
    url(r'^drffp/import/(?P<ufp_id>\d+)/$', views.permissions_import_json, name='drffp-import-ufp'),
]



# -*- coding: utf-8 -*-

"""
"""

import logging
import collections

from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.utils.encoding import smart_text
from rest_framework import serializers
from rest_framework.fields import Field

from .serializers import ModelPermissionsSerializer
from .utils import get_serializer

logger = logging.getLogger(__name__)


class ModelPermissionsField(Field):
    """Field that acts as a ModelPermissionsSerializer for relations. """
    default_error_messages = {
        'does_not_exist': 'Object with pk={value} does not exist.',
    }

    def __init__(self, serializer, **kwargs):
        self.queryset = kwargs.pop('queryset', None)

        super(ModelPermissionsField, self).__init__(**kwargs)
        self._serializer = serializer
        self._serializer_cls = None
        self.read_only = kwargs.get('read_only', None)

    @property
    def serializer(self):
        if not self._serializer_cls:
            self._serializer_cls = get_serializer(self._serializer)
        return self._serializer_cls

    def to_internal_value(self, data):
        assert self.queryset is not None or self.read_only, (
            'ModelPermissionsField field must provide a `queryset` argument, '
            'override `get_queryset`, or set read_only=`True`.'
        )

        if data is not None:
            try:
                return self.queryset.get(pk=data)
            except ObjectDoesNotExist:
                self.fail('does_not_exist', value=smart_text(data))

    def to_representation(self, obj):
        """ Represent data for the field. """
        many = isinstance(obj, collections.Iterable) \
            or isinstance(obj, models.Manager) \
            and not isinstance(obj, dict)

        assert self.serializer is not None \
            and issubclass(self.serializer, serializers.ModelSerializer), (
                "Bad serializer defined %s" % self.serializer
            )

        extra_params = {}
        if issubclass(self.serializer, ModelPermissionsSerializer):
            extra_params['cached_allowed_fields'] =\
                self.parent.cached_allowed_fields

        ser = self.serializer(obj, context=self.context, many=many,
                              **extra_params)
        return ser.data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FieldPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='name')),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType', on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name': 'field permission',
                'verbose_name_plural': 'fields permissions',
                'db_table': 'drf_field_permission',
            },
        ),
        migrations.CreateModel(
            name='FilterPermissionModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filter', models.TextField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType', on_delete=models.CASCADE)),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name': 'user filter permission',
                'verbose_name_plural': 'user filters permissions',
                'db_table': 'drf_user_filter_permissions',
            },
        ),
        migrations.CreateModel(
            name='UserFieldPermissions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('permissions', models.ManyToManyField(to='rest_framework_fine_permissions.FieldPermission', db_table='drf_user_field_permissions_permission', related_name='user_field_permissions')),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name': 'user field permission',
                'verbose_name_plural': 'user fields permissions',
                'db_table': 'drf_user_field_permissions',
            },
        ),
        migrations.AlterUniqueTogether(
            name='filterpermissionmodel',
            unique_together=set([('user', 'content_type')]),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db.models import Q
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes.models import ContentType
from rest_framework_fine_permissions.serializers import QSerializer
from rest_framework_fine_permissions.models import FilterPermissionModel
from rest_framework.filters import BaseFilterBackend
from django.db.models.query import QuerySet


class FilterPermissionBackend(BaseFilterBackend):
    """
    Filter permission backend
    """

    def filter_queryset(self, request, queryset, view):
        user = request.user
        if not user.is_superuser and not user.is_anonymous and isinstance(queryset, QuerySet):
            try:
                model = queryset.model
                ct = ContentType.objects.get_for_model(model)
                fpm = FilterPermissionModel.objects.get(user=user,
                                                        content_type=ct)
                myfilter = QSerializer(base64=True).loads(fpm.filter)
            except ObjectDoesNotExist:
                myfilter = Q()
            return queryset.filter(myfilter)
        else:
            return queryset



# -*- coding: utf-8 -*-

from django.contrib.auth.models import User
from django.test import TestCase

from rest_framework_fine_permissions.admin import UserFieldPermissionsForm
from rest_framework_fine_permissions.models import UserFieldPermissions


class TestUserFieldPermissionsForm(TestCase):

    def setUp(self):
        self.user = User.objects.create_user('test_user')

    def test_user_field_permissions_form_save(self):
        data = {
            'user': self.user.pk,
            'permissions': ['tests.account.cards', 'tests.service.name']
        }
        form = UserFieldPermissionsForm(data=data)
        self.assertTrue(form.is_bound)
        self.assertTrue(form.is_valid())
        form.save()

        self.assertIsNotNone(
            UserFieldPermissions.objects.get(pk=form.instance.pk))

    def test_user_field_permissions_form_with_conflicts(self):
        data = {
            'user': self.user.pk,
            'permissions': ['tests.account.cards', 'tests.card.account']
        }
        form = UserFieldPermissionsForm(data=data)
        self.assertTrue(form.is_bound)
        self.assertFalse(form.is_valid())

        error = str(form.errors.get('__all__')[0])
        self.assertIn('Recursive ModelPermissionsField call between', error)
        self.assertIn('tests.account.cards', error)
        self.assertIn('tests.card.account', error)



# Generated by Django 3.0.8 on 2020-09-16 10:15

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rest_framework_fine_permissions', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='UserFieldPermissions',
            name='user',
            field=models.ForeignKey(
                settings.AUTH_USER_MODEL,
                on_delete=models.CASCADE,
            )
        ),
        migrations.AlterField(
            model_name='FilterPermissionModel',
            name='user',
            field=models.ForeignKey(
                settings.AUTH_USER_MODEL,
                on_delete=models.CASCADE,
            )
        ),
    ]



# -*- coding: utf-8 -*-

"""
"""

import base64
from collections import OrderedDict
from datetime import datetime, date
import json
import time

from django.core.serializers.base import SerializationError
from django.db.models import Q
from rest_framework import serializers
from rest_framework.utils.field_mapping import get_relation_kwargs

from .models import FieldPermission


class ModelPermissionsSerializer(serializers.ModelSerializer):

    """ Permission on serializer's fields for an authenticated user. """

    def __init__(self, *args, **kwargs):
        """ Check context to retrive authenticated user. """

        self.cached_allowed_fields = kwargs.pop('cached_allowed_fields', {})
        super(ModelPermissionsSerializer, self).__init__(*args, **kwargs)

        # in case of a nested relation, we check context in meta options
        # of the nested class and set this for context
        # otherwise, the context is defined by the inherited serializer class.
        if not self.context:
            # Force to re-compute the context cached_property for drf<3.6
            try:
                del self.context
            except Exception:
                pass

            self._context = getattr(self.Meta, 'nested_context', {})
        try:
            self.user = self.context['request'].user
        except KeyError:
            self.user = self.context.get('user', None)

    def _get_user_allowed_fields(self):
        """ Retrieve all allowed field names ofr authenticated user. """
        model_name = self.Meta.model.__name__.lower()
        app_label = self.Meta.model._meta.app_label
        full_model_name = '%s.%s' % (app_label, model_name)
        permissions = self.cached_allowed_fields.get(full_model_name)

        if not permissions:
            permissions = FieldPermission.objects.filter(
                user_field_permissions__user=self.user,
                content_type__model=model_name,
                content_type__app_label=app_label
            )
            self.cached_allowed_fields[full_model_name] = permissions
        return permissions

    def get_fields(self):
        """ Calculate fields that can be accessed by authenticated user. """
        ret = OrderedDict()

        # no rights to see anything
        if not self.user:
            return ret

        # all fields that can be accessed through serializer
        fields = super(ModelPermissionsSerializer, self).get_fields()

        # superuser can see all the fields
        if self.user.is_superuser:
            return fields

        # fields that can be accessed by auhtenticated user
        allowed_fields = self._get_user_allowed_fields()
        for allowed_field in allowed_fields:
            field = fields[allowed_field.name]

            # subfields are NestedModelSerializer
            if isinstance(field, ModelPermissionsSerializer):
                # no rights on subfield's fields
                # calculate how the relation should be retrieved
                if not field.get_fields():
                    field_cls = field._related_class
                    kwargs = get_relation_kwargs(allowed_field.name,
                                                 field.info)
                    if not issubclass(field_cls,
                                      serializers.HyperlinkedRelatedField):
                        kwargs.pop('view_name', None)
                    field = field_cls(**kwargs)

            ret[allowed_field.name] = field
        return ret

    def _get_default_field_names(self, declared_fields, model_info):
        """ Return default field names for serializer. """
        return (
            [model_info.pk.name] +
            list(declared_fields.keys()) +
            list(model_info.fields.keys()) +
            list(model_info.relations.keys())
        )

    def _get_nested_class(self, nested_depth, relation_info):
        """ Define the serializer class for a relational field. """
        class NestedModelPermissionSerializer(ModelPermissionsSerializer):

            """ Default nested class for relation. """

            class Meta:
                model = relation_info.related_model
                depth = nested_depth - 1
                nested_context = self.context
                fields = '__all__'

        return NestedModelPermissionSerializer


class QSerializer():
    """
    A Q object serializer base class. Use json.
    """
    b64_enabled = False

    def __init__(self, base64=False):
        if base64:
            self.b64_enabled = True
        try:
            self.min_ts = time.mktime(datetime.min.timetuple())
        except OverflowError:
            # This Exception is thrown by some PC / MAC architectures which
            # can't work with dates before Epoch (01-01-1970). That's why, in
            # this case we take as minimal value the EPOCH (gmtime(0)).
            self.min_ts = time.mktime(time.gmtime(0))
        self.max_ts = time.mktime((3000,) + (0,) * 8)
        self.dt2ts = lambda obj: time.mktime(obj.timetuple()) if isinstance(
            obj, date) else obj

    @staticmethod
    def _is_range(qtuple):
        return qtuple[0].endswith("__range") and len(qtuple[1]) == 2

    def prepare_value(self, qtuple):
        if self._is_range(qtuple):
            qtuple[1][0] = qtuple[1][0] or self.min_ts
            qtuple[1][1] = qtuple[1][1] or self.max_ts
            qtuple[1] = (datetime.fromtimestamp(qtuple[1][0]),
                         datetime.fromtimestamp(qtuple[1][1]))
        return tuple(qtuple)

    def serialize(self, q):
        """
        Serialize a Q object into a (possibly nested) dict.
        """
        children = []
        for child in q.children:
            if isinstance(child, Q):
                children.append(self.serialize(child))
            else:
                children.append(child)
        serialized = q.__dict__
        serialized['children'] = children
        return serialized

    def deserialize(self, d):
        """
        De-serialize a Q object from a (possibly nested) dict.
        """
        children = []
        for child in d.pop('children'):
            if isinstance(child, dict):
                children.append(self.deserialize(child))
            else:
                children.append(self.prepare_value(child))
        query = Q()
        query.children = children
        query.connector = d['connector']
        query.negated = d['negated']
        if 'subtree_parents' in d:
            query.subtree_parents = d['subtree_parents']
        return query

    def dumps(self, obj):
        if not isinstance(obj, Q):
            raise SerializationError
        string = json.dumps(self.serialize(obj), default=self.dt2ts,
                            sort_keys=True,
                            indent=4, separators=(',', ': '))
        if self.b64_enabled:
            return base64.b64encode(string.encode('utf-8')).decode('utf-8')
        return string

    def loads(self, string, raw=False):
        if self.b64_enabled:
            d = json.loads(base64.b64decode(string).decode('utf-8'))
        else:
            d = json.loads(string)
        if raw:
            return d
        return self.deserialize(d)



import json

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from rest_framework_fine_permissions.models import FieldPermission,\
                                                   UserFieldPermissions


@staff_member_required
def permissions_export_json(request, ufp_id):
    ufp = get_object_or_404(UserFieldPermissions, pk=ufp_id)
    permissions = {
        'username': ufp.user.username,
        'fields_permissions': [{
            'app_label': permission.content_type.app_label,
            'model': permission.content_type.model,
            'name': permission.name
        } for permission in ufp.permissions.all()]
    }
    return HttpResponse(
        json.dumps(permissions), content_type='application/json')


@staff_member_required
def permissions_import_json(request, ufp_id=0):
    upload_file = request.FILES.get('perms_upload')

    if upload_file:
        try:
            perms = json.loads(upload_file.read().decode('utf-8'))
            username = perms.get('username')

            if ufp_id:
                ufp = get_object_or_404(UserFieldPermissions, pk=ufp_id)
                if username == ufp.user.username:
                    # Clear the old field permissions
                    ufp.permissions.clear()
                else:
                    message = 'The wrong user is defined in the imported file'
                    messages.add_message(request, messages.ERROR, message)
            else:
                user = get_object_or_404(User, username=username)
                ufp = UserFieldPermissions(user=user)
                ufp.save()

            _add_permissions(request, ufp, perms)
            ufp_id = ufp.pk
        except Exception as e:
            message = 'Error in the import : %s' % e
            messages.add_message(request, messages.ERROR, message)
    else:
        message = 'Missing imported file'
        messages.add_message(request, messages.ERROR, message)

    return redirect(
        '/admin/rest_framework_fine_permissions/userfieldpermissions/%s'
        % ufp_id)


def _add_permissions(request, ufp, perms):
    # Create the new field permissions
    fields = perms.get('fields_permissions')
    for field in fields:
        content_type = ContentType.objects.get(
            app_label=field['app_label'], model=field['model'])
        fp = FieldPermission.objects.get_or_create(
            content_type=content_type, name=field['name'])[0]
        ufp.permissions.add(fp)

    message = 'Permissions imported'
    messages.add_message(request, messages.INFO, message)



# -*- coding: utf-8 -*-

from django.test import TestCase
from django.db.models import Q
from rest_framework_fine_permissions.serializers import QSerializer
import datetime
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from rest_framework_fine_permissions.models import FilterPermissionModel
from rest_framework_fine_permissions.permissions import FilterPermission
from django.http import HttpRequest
from rest_framework_fine_permissions.filters import FilterPermissionBackend

"""

Run this test under a django application, with the following command :

python manage.py test rest_framework_fine_permissions

"""


class TestQSerializer(TestCase):
    """
    test the q serializer
    """

    def setUp(self):
        self.q1 = Q(Q(field1__contains="test") |
                    Q(rel1__field2__exact="test2") &
                    Q(Q(field3="test3") |
                      Q(field3__range=(datetime.datetime(2005, 1, 1),
                                       datetime.datetime(2005, 3, 31)))))
        self.qserializer = QSerializer(base64=True)
        self.dumps = self.qserializer.dumps(self.q1)

    def test_dumps(self):
        dumps = self.qserializer.dumps(self.q1)
        self.assertEqual(dumps, self.dumps)

    def test_loads(self):
        loads = self.qserializer.loads(self.dumps)
        self.assertEqual(self.qserializer.serialize(loads),
                         self.qserializer.serialize(self.q1))

    def test_dumps_and_loads_with_other_instance(self):
        dumps = self.qserializer.dumps(self.q1)
        self.assertEqual(dumps, self.dumps)
        qserializer2 = QSerializer(base64=True)
        loads = qserializer2.loads(dumps)
        self.assertEqual(qserializer2.serialize(loads),
                         qserializer2.serialize(self.q1))


class TestFilterPermissionModel(TestCase):
    """
    test the filter permissions model
    """

    def setUp(self):
        self.me = User.objects.create(username="morgan", password='morgan')        
        self.q = Q(Q(username='arthur') | Q(username='jean'))
        self.qserializer = QSerializer(base64=True)
        self.myfilter = self.qserializer.dumps(self.q)
        self.user_ct = ContentType.objects.get_by_natural_key("auth", "user")

    def test_create(self):
        fp = FilterPermissionModel.objects.create(user=self.me,
                                                  content_type=self.user_ct,
                                                  filter=self.myfilter)
        self.assertEqual(fp.user.username, 'morgan')
        self.assertEqual(fp.content_type.name, 'user')
        self.assertEqual(fp.filter, self.myfilter)


class TestFilterPermission(TestCase):
    """
    test filter permission
    """

    def setUp(self):
        self.filterperm = FilterPermission()
        self.request = HttpRequest()
        self.me = User.objects.create(username="morgan", password='morgan')
        self.userok = User.objects.create(username="arthur", password='arthur')
        self.not_admin = User.objects.create(username="jean", password='jean')
        self.wrong = User.objects.create(username="jojo", password='jojo')
        self.admin = User.objects.create_superuser(username="admin", password="admin", email="")
        self.user_ct = ContentType.objects.get_by_natural_key("auth", "user")
        self.q = Q(Q(username='arthur') | Q(username='jean'))
        self.qserializer = QSerializer(base64=True)
        self.myfilter = self.qserializer.dumps(self.q)
        self.fpm = FilterPermissionModel.objects.create(user=self.me,
                                                        content_type=self.user_ct,
                                                        filter=self.myfilter)
        self.request.user = self.me

    def test_superuser(self):
        self.request.user = self.admin
        res = self.filterperm.has_object_permission(self.request, None, None)
        self.assertTrue(res)

    def test_no_filter(self):
        self.request.user = self.not_admin
        res = self.filterperm.has_object_permission(self.request, None, self.userok)
        self.assertTrue(res)
        res = self.filterperm.has_object_permission(self.request, None, self.wrong)
        self.assertTrue(res)

    def test_obj_valid(self):
        res = self.filterperm.has_object_permission(self.request, None, self.userok)
        self.assertTrue(res)

    def test_obj_not_valid(self):
        res = self.filterperm.has_object_permission(self.request, None, self.wrong)
        self.assertFalse(res)


class TestFilterPermissionFilter(TestCase):
    """
    test filter
    """

    def setUp(self):
        self.filterperm = FilterPermission()
        self.me = User.objects.create(username="morgan", password='morgan')
        self.userok = User.objects.create(username="arthur", password='arthur')
        self.not_admin = User.objects.create(username="jean", password='jean')
        self.wrong = User.objects.create(username="jojo", password='jojo')
        self.admin = User.objects.create_superuser(username="admin", password="admin", email="")
        self.user_ct = ContentType.objects.get_by_natural_key("auth", "user")
        self.q = Q(Q(username='arthur') | Q(username='jean'))
        self.qserializer = QSerializer(base64=True)
        self.myfilter = self.qserializer.dumps(self.q)
        self.fpm = FilterPermissionModel.objects.create(user=self.me,
                                                        content_type=self.user_ct,
                                                        filter=self.myfilter)
        self.request = HttpRequest()
        self.request.user = self.me
        self.queryset = User.objects.all()
        self.filterbackend = FilterPermissionBackend()


    def test_manager_valid(self):
        filter = self.filterbackend.filter_queryset(self.request, self.queryset, None)
        self.assertEqual(len(filter.all()), 2)
        self.assertEqual(filter.get(username='arthur'), self.userok)

    def test_manager_not_valid(self):
        filter = self.filterbackend.filter_queryset(self.request, self.queryset, None)
        self.assertEqual(len(filter.all()), 2)
        with self.assertRaises(User.DoesNotExist):
            filter.get(username='jojo')

    def test_superuser(self):
        self.request.user = self.admin
        filter = self.filterbackend.filter_queryset(self.request, self.queryset, None)
        self.assertEqual(len(filter.all()), 5)

    def test_no_filter(self):
        self.request.user = self.not_admin
        filter = self.filterbackend.filter_queryset(self.request, self.queryset, None)
        self.assertEqual(len(filter.all()), 5)


from django.conf.urls import include, url
from django.contrib import admin

from rest_framework_fine_permissions.urls import urlpatterns as drffp_urls


urlpatterns = drffp_urls
urlpatterns += [
    url(r'^admin/', admin.site.urls),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages


with open('README.rst') as readme:
    long_description = readme.read()


with open('requirements.txt') as requirements:
    lines = requirements.readlines()
    libraries = [lib for lib in lines if not lib.startswith('-')]
    dependency_links = [link.split()[1] for link in lines if
        link.startswith('-f')]


setup(
    name='djangorestframework-fine-permissions',
    version='0.9.0',
    packages=find_packages(exclude=["tests*"]),
    install_requires=libraries,
    dependency_links=dependency_links,
    include_package_data=True,
    long_description=long_description,
    author='Arnaud Grausem',
    author_email='arnaud.grausem@unistra.fr',
    maintainer='Arnaud Grausem',
    maintainer_email='arnaud.grausem@unistra.fr',
    description='Field level permissions for Django REST Framework',
    keywords=['django', 'REST', 'rest_framework', 'permissions'],
    url='https://github.com/unistra/django-rest-framework-fine-permissions',
    download_url='https://pypi.python.org/pypi/djangorestframework-fine-permissions',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.2',
    ]
)



# -*- coding: utf-8 -*-

""" Provides new permission policies for django-rest-framework
"""

from rest_framework.permissions import DjangoModelPermissions, BasePermission
from django.contrib.contenttypes.models import ContentType
from rest_framework_fine_permissions.models import FilterPermissionModel
from django.core.exceptions import ObjectDoesNotExist
from rest_framework_fine_permissions.serializers import QSerializer


class FullDjangoModelPermissions(DjangoModelPermissions):
    """
    The request is authenticated using `django.contrib.auth` permissions.
    See: https://docs.djangoproject.com/en/dev/topics/auth/#permissions

    It ensures that the user is authenticated, and has the appropriate
    `view`/`add`/`change`/`delete` permissions on the model.

    This permission can only be applied against view classes that provide a
    `.model` or `.queryset` attribute.
    """

    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s'],
        'OPTIONS': [],
        'HEAD': ['%(app_label)s.view_%(model_name)s'],
        'POST': ['%(app_label)s.add_%(model_name)s'],
        'PUT': ['%(app_label)s.change_%(model_name)s'],
        'PATCH': ['%(app_label)s.change_%(model_name)s'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
    }


class FilterPermission(BasePermission):
    """
    filter permission
    """

    def has_object_permission(self, request, view, obj):
        """
        check filter permissions
        """

        user = request.user

        if not user.is_superuser and not user.is_anonymous:
            valid = False
            try:
                ct = ContentType.objects.get_for_model(obj)
                fpm = FilterPermissionModel.objects.get(user=user,
                                                        content_type=ct)
                myq = QSerializer(base64=True).loads(fpm.filter)
                try:
                    myobj = obj.__class__.objects.filter(myq).distinct().get(pk=obj.pk)
                    if myobj:
                        valid = True
                except ObjectDoesNotExist:
                    valid = False
            except ObjectDoesNotExist:
                valid = True
            finally:
                return valid
        else:
            return True



#!/usr/bin/env python

from django.core import management
from django.core.management.base import BaseCommand, CommandError
from django.core import serializers
from rest_framework_fine_permissions.models import UserFieldPermissions, FieldPermission
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
import json
from optparse import make_option
import sys
from django.contrib.contenttypes.models import ContentType



class Command(BaseCommand):

    help = ("Loads the user's fieds permissions into the database from a fixture of the given user ")
    args = 'json file'


    option_list = BaseCommand.option_list + (
        make_option('-u', '--user', help='Specify another user'),
    )

    def handle(self, *args, **options):
        """
        dump fields permissions for a user
        """

        def get_user(username):
            try:
                return User.objects.get(username=username)
            except ObjectDoesNotExist as e:
                raise CommandError("This user doesn't exist in the database")

        def add_permissions(user_field_permissions, content_type, name):

            p = None
            try:
                p = FieldPermission.objects.get(content_type=content_type, name=name)
            except ObjectDoesNotExist:
                p = FieldPermission(content_type=content_type, name=name)
                p.save()
            finally:
                user_field_permissions.permissions.add(p)



        if len(args) !=1:
             raise CommandError("Specifies a json file created by the fine_permissions_dump command")
        else:
            try:


                with open(args[0], 'r') as json_file:
                    myjson = json.load(json_file)

                    user = get_user(options.get('user')) if options['user'] else get_user(myjson['username'])
                    fields_permissions = myjson['fields_permissions']

                    user_field_permissions = UserFieldPermissions(user=user)
                    user_field_permissions.save()

                    for f in fields_permissions:
                        content_type = ContentType.objects.get(app_label=f["app_label"], model=f["model"])
                        add_permissions(user_field_permissions, content_type, f['name'])


            except Exception as e:
                raise CommandError(e)




from django.db import models


class TestUser(models.Model):
    username = models.CharField(max_length=255)


class Account(models.Model):

    user = models.OneToOneField(TestUser, on_delete=models.CASCADE)
    expired_date = models.DateTimeField()

    @property
    def full_name(self):
        return '{0.first_name} {0.last_name}'.format(self.user)

    def is_expired(self):
        from datetime import datetime
        return self.expired_date > datetime.now()

    @property
    def services(self):
        return Service.objects.filter(cards__account__pk=self.pk)


class Service(models.Model):

    name = models.CharField(max_length=100)


class Card(models.Model):

    account = models.ForeignKey(Account, related_name='cards',
                                on_delete=models.CASCADE)
    services = models.ManyToManyField(Service, related_name='cards')
